"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validatePackageFile = exports.validatePackageFileInBrowser = exports.parsePackageFileJSON = exports.loadPackageFileFromDisk = exports.validatePackageSlug = exports.validateCatalogSlug = exports.compatibilityToString = exports.nextVersion = exports.diffCompatibility = exports.compareConfigObjects = exports.compareSchema = exports.compareSchemas = exports.comparePackages = exports.leastCompatible = exports.DifferenceType = exports.Compability = void 0;
const semver_1 = require("semver");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const ajv_1 = __importDefault(require("ajv"));
const cross_fetch_1 = __importDefault(require("cross-fetch"));
var Compability;
(function (Compability) {
    Compability[Compability["Identical"] = 0] = "Identical";
    Compability[Compability["MinorChange"] = 1] = "MinorChange";
    Compability[Compability["CompatibleChange"] = 2] = "CompatibleChange";
    Compability[Compability["BreakingChange"] = 3] = "BreakingChange";
})(Compability = exports.Compability || (exports.Compability = {}));
var DifferenceType;
(function (DifferenceType) {
    DifferenceType["REMOVE_SCHEMA"] = "REMOVE_SCHEMA";
    DifferenceType["ADD_SCHEMA"] = "ADD_SCHEMA";
    DifferenceType["CHANGE_PACKAGE_DISPLAY_NAME"] = "CHANGE_PACKAGE_DISPLAY_NAME";
    DifferenceType["CHANGE_PACKAGE_DESCRIPTION"] = "CHANGE_PACKAGE_DESCRIPTION";
    DifferenceType["CHANGE_SOURCE"] = "CHANGE_SOURCE";
    DifferenceType["CHANGE_SOURCE_UPDATE_HASH"] = "CHANGE_SOURCE_UPDATE_HASH";
    DifferenceType["ADD_PROPERTY"] = "ADD_PROPERTY";
    DifferenceType["REMOVE_PROPERTY"] = "REMOVE_PROPERTY";
    DifferenceType["CHANGE_PROPERTY_TYPE"] = "CHANGE_PROPERTY_TYPE";
    DifferenceType["CHANGE_PROPERTY_FORMAT"] = "CHANGE_PROPERTY_FORMAT";
    DifferenceType["CHANGE_PROPERTY_DESCRIPTION"] = "CHANGE_PROPERTY_DESCRIPTION";
    DifferenceType["CHANGE_GENERATED_BY"] = "CHANGE_GENERATED_BY";
    DifferenceType["CHANGE_UPDATED_DATE"] = "CHANGE_UPDATED_DATE";
    DifferenceType["CHANGE_VERSION"] = "CHANGE_VERSION";
    DifferenceType["CHANGE_README_MARKDOWN"] = "CHANGE_README_MARKDOWN";
    DifferenceType["CHANGE_LICENSE_MARKDOWN"] = "CHANGE_LICENSE_MARKDOWN";
    DifferenceType["CHANGE_README_FILE"] = "CHANGE_README_FILE";
    DifferenceType["CHANGE_LICENSE_FILE"] = "CHANGE_LICENSE_FILE";
    DifferenceType["CHANGE_WEBSITE"] = "CHANGE_WEBSITE";
    DifferenceType["CHANGE_CONTACT_EMAIL"] = "CHANGE_CONTACT_EMAIL";
})(DifferenceType = exports.DifferenceType || (exports.DifferenceType = {}));
function leastCompatible(a, b) {
    if (a === b)
        return a;
    if (a < b)
        return b;
    return a;
}
exports.leastCompatible = leastCompatible;
function comparePackages(packageA, packageB) {
    let response = [];
    if (packageA.description !== packageB.description)
        response.push({
            type: DifferenceType.CHANGE_PACKAGE_DESCRIPTION,
            pointer: "#"
        });
    if (packageA.displayName !== packageB.displayName)
        response.push({
            type: DifferenceType.CHANGE_PACKAGE_DISPLAY_NAME,
            pointer: "#"
        });
    if (packageA.version !== packageB.version) {
        response.push({
            type: DifferenceType.CHANGE_VERSION,
            pointer: "#"
        });
    }
    if (packageA.readmeFile !== packageB.readmeFile) {
        response.push({
            type: DifferenceType.CHANGE_README_FILE,
            pointer: "#"
        });
    }
    if (packageA.website !== packageB.website) {
        response.push({
            type: DifferenceType.CHANGE_WEBSITE,
            pointer: "#"
        });
    }
    if (packageA.contactEmail !== packageB.contactEmail) {
        response.push({
            type: DifferenceType.CHANGE_CONTACT_EMAIL,
            pointer: "#"
        });
    }
    if (packageA.readmeMarkdown !== packageB.readmeMarkdown) {
        response.push({
            type: DifferenceType.CHANGE_README_MARKDOWN,
            pointer: "#"
        });
    }
    if (packageA.licenseFile !== packageB.licenseFile) {
        response.push({
            type: DifferenceType.CHANGE_LICENSE_FILE,
            pointer: "#"
        });
    }
    if (packageA.licenseMarkdown !== packageB.licenseMarkdown) {
        response.push({
            type: DifferenceType.CHANGE_LICENSE_MARKDOWN,
            pointer: "#"
        });
    }
    if (packageA.generatedBy !== packageB.generatedBy)
        response.push({ type: DifferenceType.CHANGE_GENERATED_BY, pointer: "#" });
    if (packageA.updatedDate.getTime() !== packageB.updatedDate.getTime())
        response.push({ type: DifferenceType.CHANGE_UPDATED_DATE, pointer: "#" });
    response = response.concat(compareSchemas(packageA.schemas, packageB.schemas));
    return response;
}
exports.comparePackages = comparePackages;
function compareSchemas(priorSchemas, newSchemas) {
    let response = [];
    for (const schemaA of priorSchemas) {
        let found = false;
        for (const schemaB of newSchemas) {
            if (schemaA.title !== schemaB.title)
                continue;
            found = true;
            response = response.concat(compareSchema(schemaA, schemaB, "#"));
        }
        if (!found)
            response.push({
                type: DifferenceType.REMOVE_SCHEMA,
                pointer: ""
            });
    }
    return response;
}
exports.compareSchemas = compareSchemas;
function compareSchema(priorSchema, newSchema, pointer = "#") {
    let response = [];
    pointer += "/" + newSchema.title;
    if (priorSchema.description !== newSchema.description)
        response.push({
            type: DifferenceType.CHANGE_PROPERTY_DESCRIPTION,
            pointer
        });
    if (Array.isArray(priorSchema.type) && Array.isArray(newSchema.type)) {
        if (!priorSchema.type.every((v) => { var _a; return ((_a = newSchema.type) === null || _a === void 0 ? void 0 : _a.indexOf(v)) !== -1; }))
            response.push({ type: DifferenceType.CHANGE_PROPERTY_TYPE, pointer });
    }
    else if (!Array.isArray(priorSchema.type) && !Array.isArray(newSchema.type)) {
        if (priorSchema.type !== newSchema.type)
            response.push({ type: DifferenceType.CHANGE_PROPERTY_TYPE, pointer });
    }
    else {
        response.push({ type: DifferenceType.CHANGE_PROPERTY_TYPE, pointer });
    }
    if (priorSchema.type === "string" && priorSchema.format !== newSchema.format)
        response.push({ type: DifferenceType.CHANGE_PROPERTY_FORMAT, pointer });
    if (priorSchema.type === "object") {
        if (priorSchema.properties == null)
            throw new Error("Prior Schema property type is object, but has no properties");
        if (newSchema.properties == null)
            throw new Error("New Schema property type is object, but has no properties");
        for (const priorKey of Object.keys(priorSchema.properties)) {
            const propertyPointer = pointer + "/properties";
            const newKeys = Object.keys(newSchema.properties);
            if (newKeys.indexOf(priorKey) === -1) {
                response.push({
                    type: DifferenceType.REMOVE_PROPERTY,
                    pointer: pointer
                });
                break;
            }
            const priorProperty = priorSchema.properties[priorKey];
            const newProperty = newSchema.properties[priorKey];
            response = response.concat(compareSchema(priorProperty, newProperty, propertyPointer));
        }
        for (const newKey of Object.keys(newSchema.properties)) {
            const priorKeys = Object.keys(priorSchema.properties);
            const propertyPointer = pointer + "/properties/" + newKey;
            if (priorKeys.indexOf(newKey) === -1) {
                response.push({
                    type: DifferenceType.ADD_PROPERTY,
                    pointer: propertyPointer
                });
            }
        }
    }
    if (priorSchema.source == null && newSchema.source != null) {
        response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
    }
    else if (newSchema.source == null && priorSchema.source != null) {
        response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
    }
    else if (priorSchema.source != null && newSchema.source != null) {
        if (priorSchema.source.type !== newSchema.source.type) {
            response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
        }
        else if (priorSchema.source.uri !== newSchema.source.uri) {
            response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
        }
        else {
            const configComparison = compareConfigObjects(priorSchema.source.configuration, newSchema.source.configuration);
            if (!configComparison)
                response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
        }
        if (priorSchema.source.lastUpdateHash !== newSchema.source.lastUpdateHash &&
            priorSchema.source.lastUpdateHash != null) {
            response.push({ type: DifferenceType.CHANGE_SOURCE_UPDATE_HASH, pointer: pointer });
        }
    }
    return response;
}
exports.compareSchema = compareSchema;
function compareConfigObjects(priorObject, newObject) {
    if (priorObject == null && newObject == null)
        return true;
    if ((priorObject == null && newObject != null) || (priorObject != null && newObject == null))
        return false;
    const newKeys = Object.keys(newObject);
    for (const priorKey of Object.keys(priorObject)) {
        if (newKeys.indexOf(priorKey) === -1)
            return false;
        const priorValue = priorObject[priorKey];
        const newValue = newObject[priorKey];
        if (typeof priorValue !== typeof newValue)
            return false;
        if (typeof priorValue === "object" && typeof newValue === "object") {
            if (!compareConfigObjects(priorValue, newValue))
                return false;
        }
        else if (priorValue !== newValue)
            return false;
    }
    return true;
}
exports.compareConfigObjects = compareConfigObjects;
function diffCompatibility(diffs) {
    let returnValue = Compability.Identical;
    diffs.forEach((d) => {
        switch (d.type) {
            case DifferenceType.REMOVE_PROPERTY:
            case DifferenceType.REMOVE_SCHEMA:
            case DifferenceType.CHANGE_PROPERTY_FORMAT:
            case DifferenceType.CHANGE_PROPERTY_TYPE:
                returnValue = Compability.BreakingChange;
                break;
            case DifferenceType.ADD_PROPERTY:
            case DifferenceType.ADD_SCHEMA:
                returnValue = Math.max(returnValue, Compability.CompatibleChange);
                break;
            case DifferenceType.CHANGE_PACKAGE_DESCRIPTION:
            case DifferenceType.CHANGE_PACKAGE_DISPLAY_NAME:
            case DifferenceType.CHANGE_PROPERTY_DESCRIPTION:
            case DifferenceType.CHANGE_SOURCE:
            case DifferenceType.CHANGE_README_MARKDOWN:
            case DifferenceType.CHANGE_LICENSE_MARKDOWN:
            case DifferenceType.CHANGE_WEBSITE:
            case DifferenceType.CHANGE_CONTACT_EMAIL:
            case DifferenceType.CHANGE_VERSION:
                returnValue = Math.max(returnValue, Compability.MinorChange);
                break;
            case DifferenceType.CHANGE_SOURCE_UPDATE_HASH:
            case DifferenceType.CHANGE_GENERATED_BY:
            case DifferenceType.CHANGE_UPDATED_DATE:
            case DifferenceType.CHANGE_README_FILE:
            case DifferenceType.CHANGE_LICENSE_FILE:
                break;
            default:
                throw new Error("Diff type " + d.type + " not mapped");
        }
    });
    return returnValue;
}
exports.diffCompatibility = diffCompatibility;
function nextVersion(currentVersion, diffCompatibility) {
    const copy = new semver_1.SemVer(currentVersion.version);
    switch (diffCompatibility) {
        case Compability.BreakingChange:
            return copy.inc("major");
        case Compability.CompatibleChange:
            return copy.inc("minor");
        case Compability.MinorChange:
            return copy.inc("patch");
        case Compability.Identical:
            return copy;
        default:
            throw new Error("Unrecognized compability " + diffCompatibility);
    }
}
exports.nextVersion = nextVersion;
function compatibilityToString(compatibility) {
    switch (compatibility) {
        case Compability.BreakingChange:
            return "breaking";
        case Compability.CompatibleChange:
            return "compatibile";
        case Compability.MinorChange:
            return "minor";
        case Compability.Identical:
            return "no";
        default:
            throw new Error("Compatibility " + compatibility + " not recognized");
    }
}
exports.compatibilityToString = compatibilityToString;
function validateCatalogSlug(slug) {
    const regExp = /^[a-z\d](?:[a-z\d]|-(?=[a-z\d])){0,38}$/i;
    if (slug === undefined)
        return false;
    return !!slug.match(regExp);
}
exports.validateCatalogSlug = validateCatalogSlug;
function validatePackageSlug(slug) {
    const regExp = /^[a-z0-9]+(?:(?:(?:[._]|__|[-]*)[a-z0-9]+)+)?$/;
    if (slug === undefined)
        return false;
    return !!slug.match(regExp);
}
exports.validatePackageSlug = validatePackageSlug;
function loadPackageFileFromDisk(packageFilePath) {
    if (!fs_1.default.existsSync(packageFilePath))
        throw new Error("FILE_NOT_FOUND");
    let packageFileAbsolutePath;
    if (path_1.default.isAbsolute(packageFilePath)) {
        packageFileAbsolutePath = path_1.default.dirname(packageFilePath);
    }
    else {
        packageFileAbsolutePath = process.cwd() + path_1.default.sep + path_1.default.dirname(packageFilePath);
    }
    let packageFile;
    const packageFileContents = fs_1.default.readFileSync(packageFilePath).toString();
    validatePackageFile(packageFileContents);
    try {
        packageFile = parsePackageFileJSON(packageFileContents);
    }
    catch (error) {
        throw new Error("PACKAGE_PARSE_ERROR: " + error.message);
    }
    if (packageFile.readmeFile != null) {
        const readmeFileAbsolutePath = packageFileAbsolutePath + path_1.default.sep + packageFile.readmeFile;
        if (!fs_1.default.existsSync(readmeFileAbsolutePath)) {
            throw new Error("README_FILE_NOT_FOUND: " + readmeFileAbsolutePath);
        }
        packageFile.readmeMarkdown = fs_1.default.readFileSync(readmeFileAbsolutePath).toString();
    }
    if (packageFile.licenseFile != null) {
        const licenseFileAbsolutePath = packageFileAbsolutePath + path_1.default.sep + packageFile.licenseFile;
        if (!fs_1.default.existsSync(licenseFileAbsolutePath)) {
            throw new Error("LICENSE_FILE_NOT_FOUND: " + licenseFileAbsolutePath);
        }
        packageFile.licenseMarkdown = fs_1.default.readFileSync(licenseFileAbsolutePath).toString();
    }
    return packageFile;
}
exports.loadPackageFileFromDisk = loadPackageFileFromDisk;
function parsePackageFileJSON(packageFileString) {
    try {
        const packageFile = JSON.parse(packageFileString, (key, value) => {
            if (key !== "updatedDate" && key !== "createdAt" && key !== "updatedAt")
                return value;
            return new Date(Date.parse(value));
        });
        return packageFile;
    }
    catch (error) {
        throw new Error("ERROR_PARSING_PACKAGE_FILE - " + error.message);
    }
}
exports.parsePackageFileJSON = parsePackageFileJSON;
async function validatePackageFileInBrowser(packageFile) {
    const ajv = new ajv_1.default({
        format: false
    });
    let packageSchemaFile;
    const response = await cross_fetch_1.default("/docs/datapm-package-file-schema-v1.json");
    if (response.status > 199 && response.status < 300) {
        packageSchemaFile = await response.text();
    }
    else {
        throw new Error("ERROR_FINDING_PACKAGE_SCHEMA_FILE " + response.status);
    }
    let schemaObject;
    try {
        schemaObject = JSON.parse(packageSchemaFile);
    }
    catch (error) {
        throw new Error("ERROR_PARSING_PACKAGE_SCHEMA_FILE: " + error.message);
    }
    if (!ajv.validateSchema(schemaObject)) {
        throw new Error("ERROR_READING_SCHEMA");
    }
    let packageFileObject;
    try {
        packageFileObject = JSON.parse(packageFile);
    }
    catch (error) {
        throw new Error("ERROR_PARSING_PACKAGE_FILE - " + error.message);
    }
    const ajvResponse = ajv.validate(schemaObject, packageFileObject);
    if (!ajvResponse) {
        throw new Error("INVALID_PACKAGE_FILE_SCHEMA: " + JSON.stringify(ajv.errors));
    }
}
exports.validatePackageFileInBrowser = validatePackageFileInBrowser;
function validatePackageFile(packageFile) {
    let packageFileObject;
    try {
        packageFileObject = JSON.parse(packageFile);
    }
    catch (error) {
        throw new Error("ERROR_PARSING_PACKAGE_FILE - " + error.message);
    }
    const ajv = new ajv_1.default({
        format: false
    });
    let packageSchemaFile;
    try {
        const pathToDataPmLib = require.resolve("datapm-lib").replace(path_1.default.sep + "src" + path_1.default.sep + "main.js", "");
        packageSchemaFile = fs_1.default.readFileSync(path_1.default.join(pathToDataPmLib, "packageFileSchema.json"), "utf8");
    }
    catch (error) {
        try {
            packageSchemaFile = fs_1.default.readFileSync("node_modules" + path_1.default.sep + "datapm-lib" + path_1.default.sep + "packageFileSchema.json", "utf8");
        }
        catch (error) {
            try {
                packageSchemaFile = fs_1.default.readFileSync("packageFileSchema.json", "utf8");
            }
            catch (error) {
                packageSchemaFile = fs_1.default.readFileSync(path_1.default.join("..", "lib", "packageFileSchema.json"), "utf8");
            }
        }
    }
    let schemaObject;
    try {
        schemaObject = JSON.parse(packageSchemaFile);
    }
    catch (error) {
        throw new Error("ERROR_PARSING_PACKAGE_SCHEMA_FILE: " + error.message);
    }
    if (!ajv.validateSchema(schemaObject)) {
        throw new Error("ERROR_READING_SCHEMA");
    }
    const response = ajv.validate(schemaObject, packageFileObject);
    if (!response) {
        throw new Error("INVALID_PACKAGE_FILE_SCHEMA: " + JSON.stringify(ajv.errors));
    }
}
exports.validatePackageFile = validatePackageFile;
//# sourceMappingURL=PackageUtil.js.map