"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PackageUtil_1 = require("../src/PackageUtil");
const semver_1 = require("semver");
const chai_1 = require("chai");
describe("Checking VersionUtil", () => {
    it("Compatibility ENUM Order", () => {
        chai_1.expect(PackageUtil_1.leastCompatible(PackageUtil_1.Compability.Identical, PackageUtil_1.Compability.BreakingChange)).equal(PackageUtil_1.Compability.BreakingChange);
    });
    it("No change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.Identical);
        chai_1.expect(oldVersion.version).equal("1.0.3");
        chai_1.expect(newVersion.version).equal("1.0.3");
    });
    it("Minor change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.MinorChange);
        chai_1.expect(oldVersion.version).equal("1.0.3");
        chai_1.expect(newVersion.version).equal("1.0.4");
    });
    it("Compatible change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.CompatibleChange);
        chai_1.expect(oldVersion.version).equal("1.0.3");
        chai_1.expect(newVersion.version).equal("1.1.0");
    });
    it("Breaking change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.BreakingChange);
        chai_1.expect(oldVersion.version).equal("1.0.3");
        chai_1.expect(newVersion.version).equal("2.0.0");
    });
    it("Simple property schema comparison", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "string",
            format: "date-time"
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "string",
            format: "date-time"
        };
        chai_1.expect(PackageUtil_1.compareSchema(schemaA1, schemaA2).length).equal(0);
        schemaA2.format = "date";
        const changeTitle = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(changeTitle[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_PROPERTY_FORMAT);
    });
    it("Type arrays vs singluar values", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "object",
            properties: {
                string: { title: "string", type: "string" },
                number: { title: "number", type: "number" }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "object",
            properties: {
                string: { title: "string", type: "string" },
                number: { title: "number", type: "number" }
            }
        };
        const diff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(diff.length).equal(0);
        schemaA1.properties.string.type = ["string", "number"];
        const arrayVsNotDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(arrayVsNotDiff.length).equal(1);
        chai_1.expect(arrayVsNotDiff[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_PROPERTY_TYPE);
        schemaA2.properties.string.type = ["string", "number"];
        const equalDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(equalDiff.length).equal(0);
    });
    it("Object schema comparison", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "object",
            properties: {
                string: { title: "string", type: "string" },
                number: { title: "number", type: "number" }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "object",
            properties: {
                string: { title: "string", type: "string" },
                number: { title: "number", type: "number" }
            }
        };
        const diff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(diff.length).equal(0);
        schemaA2.properties.boolean = { title: "boolean", type: "boolean" };
        const compatibleChange = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(compatibleChange.length).equal(1);
        chai_1.expect(compatibleChange[0].type).equal(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        schemaA1.properties.date = {
            title: "date",
            type: "string",
            format: "date"
        };
        const addPropertyDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(addPropertyDiff.length).equal(2);
        const propertyRemoved = addPropertyDiff.find((d) => d.type === PackageUtil_1.DifferenceType.ADD_PROPERTY);
        chai_1.expect(propertyRemoved != null).equal(true);
        schemaA1.properties.boolean = { title: "boolean", type: "boolean" };
        schemaA2.properties.date = {
            title: "date",
            type: "string",
            format: "date"
        };
        const finalDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(finalDiff).length(0);
    });
    it("Diff compatibility testing", () => {
        chai_1.expect(PackageUtil_1.nextVersion(new semver_1.SemVer("1.0.2"), PackageUtil_1.Compability.BreakingChange).version).equal(new semver_1.SemVer("2.0.0").version);
        chai_1.expect(PackageUtil_1.nextVersion(new semver_1.SemVer("1.0.3"), PackageUtil_1.Compability.CompatibleChange).version).equal(new semver_1.SemVer("1.1.0").version);
        chai_1.expect(PackageUtil_1.nextVersion(new semver_1.SemVer("1.0.3"), PackageUtil_1.Compability.MinorChange).version).equal(new semver_1.SemVer("1.0.4").version);
    });
    it("Nested Objects Comparison", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "object",
            properties: {
                object: {
                    title: "object",
                    type: "object",
                    properties: {
                        string1: { type: "string" }
                    }
                },
                number: { title: "number", type: "number" }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "object",
            properties: {
                object: {
                    title: "object",
                    type: "object",
                    properties: {
                        string1: { type: "string" }
                    }
                },
                number: { title: "number", type: "number" }
            }
        };
        const firstDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(firstDiff).length(0);
        chai_1.expect(PackageUtil_1.diffCompatibility(firstDiff)).equal(PackageUtil_1.Compability.Identical);
        schemaA2.properties.object.properties.string2 = { type: "string" };
        const compatibleDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(compatibleDiff).length(1);
        chai_1.expect(compatibleDiff[0].type).equal(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        chai_1.expect(compatibleDiff[0].pointer).equal("#/SchemaA/properties/object/properties/string2");
        const compatibleComparison = PackageUtil_1.diffCompatibility(compatibleDiff);
        chai_1.expect(compatibleComparison).equal(PackageUtil_1.Compability.CompatibleChange);
        schemaA1.properties.object.properties.string3 = { type: "string" };
        const breakingDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(breakingDiff).length(2);
        chai_1.expect(breakingDiff[0].type).equal(PackageUtil_1.DifferenceType.REMOVE_PROPERTY);
        chai_1.expect(breakingDiff[0].pointer).equal("#/SchemaA/properties/object");
        chai_1.expect(breakingDiff[1].type).equal(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        chai_1.expect(breakingDiff[1].pointer).equal("#/SchemaA/properties/object/properties/string2");
        const breakingChange = PackageUtil_1.diffCompatibility(breakingDiff);
        chai_1.expect(breakingChange).equal(PackageUtil_1.Compability.BreakingChange);
        schemaA1.properties.object.properties.string2 = { type: "string" };
        schemaA2.properties.object.properties.string3 = { type: "string" };
        const finalDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(finalDiff).length(0);
        chai_1.expect(PackageUtil_1.diffCompatibility(finalDiff)).equal(PackageUtil_1.Compability.Identical);
    });
    it("Catalog slug validation", () => {
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a")).equal(true);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("0")).equal(true);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a-b")).equal(true);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a-b-123")).equal(true);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a".repeat(39))).equal(true);
        chai_1.expect(PackageUtil_1.validateCatalogSlug(undefined)).equal(false);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("")).equal(false);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a_b")).equal(false);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a--b")).equal(false);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a-b-")).equal(false);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("-a-b")).equal(false);
        chai_1.expect(PackageUtil_1.validateCatalogSlug("a".repeat(40))).equal(false);
    });
    it("Package slug validation", () => {
        chai_1.expect(PackageUtil_1.validatePackageSlug("a")).equal(true);
        chai_1.expect(PackageUtil_1.validatePackageSlug("0")).equal(true);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a.b")).equal(true);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a--b")).equal(true);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a__b")).equal(true);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a__b----c.123")).equal(true);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a".repeat(100))).equal(true);
        chai_1.expect(PackageUtil_1.validatePackageSlug(undefined)).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug(".")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("-")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("_")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a@b")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a.")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a..b")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a-")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a_")).equal(false);
        chai_1.expect(PackageUtil_1.validatePackageSlug("a___c")).equal(false);
    });
    it("Compare identical", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "string",
            format: "date-time",
            source: {
                type: "test",
                uri: "http://datapm.io/test",
                configuration: {},
                lastUpdateHash: "abc123"
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "string",
            format: "date-time",
            source: {
                type: "test",
                uri: "http://datapm.io/test",
                configuration: {},
                lastUpdateHash: "abc123"
            }
        };
        if (schemaA2.source == null) {
            throw new Error("source should not be null");
        }
        chai_1.expect(PackageUtil_1.compareSchema(schemaA1, schemaA2).length).equal(0);
        schemaA2.source.uri = "https://somethingelse.datapm.io";
        const diffs = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(diffs[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_SOURCE);
        schemaA2.source.uri = "http://datapm.io/test";
        schemaA2.source.lastUpdateHash = "test1234";
        const diffs2 = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        chai_1.expect(diffs2[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_SOURCE_UPDATE_HASH);
    });
    it("Package File updated dates", function () {
        const packageFileA = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test"
        };
        const packageFileB = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test"
        };
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === "CHANGE_UPDATED_DATE")).equal(false);
        packageFileB.updatedDate = new Date(new Date().getTime() - 100);
        const diff = PackageUtil_1.comparePackages(packageFileA, packageFileB);
        chai_1.expect(diff.some((d) => d.type === "CHANGE_UPDATED_DATE")).equal(true);
    });
    it("Package File updated versions", function () {
        const packageFileA = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test"
        };
        const packageFileB = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test"
        };
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === "CHANGE_VERSION")).equal(false);
        packageFileB.version = "1.0.1";
        const diff = PackageUtil_1.comparePackages(packageFileA, packageFileB);
        chai_1.expect(diff.some((d) => d.type === "CHANGE_VERSION")).equal(true);
    });
    it("Package File updated readme", function () {
        const packageFileA = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            readmeMarkdown: "Some readme content"
        };
        const packageFileB = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            readmeMarkdown: packageFileA.readmeMarkdown
        };
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_README_MARKDOWN)).equal(false);
        packageFileB.readmeFile = "some-new-file.README.md";
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_README_FILE)).equal(true);
        packageFileB.readmeMarkdown = "other content";
        const diff = PackageUtil_1.comparePackages(packageFileA, packageFileB);
        chai_1.expect(diff.some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_README_MARKDOWN)).equal(true);
    });
    it("Package File updated license", function () {
        const packageFileA = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            licenseMarkdown: "Some content"
        };
        const packageFileB = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            licenseMarkdown: packageFileA.licenseMarkdown
        };
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_LICENSE_MARKDOWN)).equal(false);
        packageFileB.licenseFile = "some-new-file.LICENSE.md";
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_LICENSE_FILE)).equal(true);
        packageFileB.licenseMarkdown = "other content";
        const diff = PackageUtil_1.comparePackages(packageFileA, packageFileB);
        chai_1.expect(diff.some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_LICENSE_MARKDOWN)).equal(true);
    });
    it("Package File updated contact email", function () {
        const packageFileA = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            readmeMarkdown: "Some readme content",
            contactEmail: "test@test.com"
        };
        const packageFileB = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            readmeMarkdown: packageFileA.readmeMarkdown,
            contactEmail: packageFileA.contactEmail
        };
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_CONTACT_EMAIL)).equal(false);
        packageFileB.contactEmail = "testb@test.com";
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_CONTACT_EMAIL)).equal(true);
    });
    it("Package File updated website", function () {
        const packageFileA = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            readmeMarkdown: "Some readme content",
            contactEmail: "test@test.com",
            website: "https://dreamingwell.com"
        };
        const packageFileB = {
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            readmeMarkdown: packageFileA.readmeMarkdown,
            contactEmail: packageFileA.contactEmail,
            website: packageFileA.website
        };
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_WEBSITE)).equal(false);
        packageFileB.website = "https://datapm.io";
        chai_1.expect(PackageUtil_1.comparePackages(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_WEBSITE)).equal(true);
    });
});
//# sourceMappingURL=PackageComparison.spec.js.map